#!/usr/bin/env python
# 
# EHE Level3 for IC40
#
from I3Tray  import *
from I3Units import *
from math    import *
from os.path import expandvars
from glob    import glob
import os

workspace = expandvars("$I3_BUILD")
indir  = "/data/exp/IceCube/2008/filtered/level2/0702/"
outdir = "/net/user/mio/RealData/2008/level3/i3_files/0702/"
outrootdir = "/net/user/mio/RealData/2008/level3/root_files/0702/"
gcdfile = indir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level2_GCD.i3.gz"
inphysfile = indir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level2_Part00000000.i3.gz"
outphysfile = outdir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level3_Part00000000.i3.gz"
outrootfile = outrootdir + "PFFilt_PhysicsTrig_PhysicsFiltering_Run00111266_Level3_Part00000000.root"
outbdlfile = outdir + "BadDomList"

if len(sys.argv) > 1:
	gcdfile = sys.argv[1]
if len(sys.argv) > 2:
	inphysfile = sys.argv[2]
if len(sys.argv) > 3:
	outphysfile = sys.argv[3]
if len(sys.argv) > 4:
	outrootfile = sys.argv[4]
if len(sys.argv) > 5:
	outbdlfile = sys.argv[5]

load("libicetray")
load("libdataclasses")
load("libphys-services")
load("libdataio")
load("libjebclasses")
load("libDOMcalibrator")
load("libDomTools")
load("libBadDomList")
#load("librealtree-module")
load("libanalysis-tree")
# Filter module
load("libicepick");
load("libfilter-tools");
load("libpfclasses");
load("libjebclasses");
# EHE module
load("libophelia")
load("libportia")
load("libehe-star")
# IceTop module
load("libtoprec")

tray = I3Tray()
#**************************************************************
# Read Level1 i3 files
#**************************************************************
tray.AddService("I3ReaderServiceFactory","reader")(
    ("FilenameList", [gcdfile,inphysfile]),
    ("SkipKeys", ["CleanInIceRawData",
		  "HLCInIceRawData",
		  "CleanIceTopRawData",
		  "CalibratedATWD",
		  "CalibratedFADC",
		  "SummaryPulseInfo",
		  "SelectedCombinedPulses",
		  "PoleIcLinefit",
		  "PoleIcLinefitParams",
		  "PoleCombinedLinefit",
		  "PoleCombinedLinefitParams",
		  "PoleCascadeLinefit",
		  "PoleCascadeLinefitParams",
		  "PoleToI",
		  "PoleToIParams",
		  "PoleTrackLlhFit",
		  "PoleTrackLlhFitFitParams",
		  "PoleInvertedTrackLlhFit",
		  "PoleInvertedTrackLlhFitFitParams",
		  "PoleTrackLlhFitOrig",
		  "PoleTrackLlhFitFitParamsOrig",
		  "PoleInvertedTrackLlhFitOrig",
		  "PoleInvertedTrackLlhFitFitParamsOrig",
		  "PoleJamsCluster",
		  "PoleJamsClusterParams",
		  "PoleJamsClusterVect",
		  "PoleJamsClusterVectParams",
		  "PoleJamsFit",
		  "PoleJamsFitParams",
		  "PoleJamsQualityFit",
		  "PoleJamsQualityFitParams",
		  "TWREventHeader0",
		  "TWREventHeader1",
		  "TWREventHeader2",
		  "TWREventHeader3",
		  "TWREventHeader4",
		  "TWREventHeader5",
		  "TWREventHeader6",
		  "TWREventHeader7",
		  "TWREventHeader8",
		  "TWREventHeader9",
		  "TWREventHeader10",
		  "TWRTriggerHierarchy0",
		  "TWRTriggerHierarchy1",
		  "TWRTriggerHierarchy2",
		  "TWRTriggerHierarchy3",
		  "TWRTriggerHierarchy4",
		  "TWRTriggerHierarchy5",
		  "TWRTriggerHierarchy6",
		  "TWRTriggerHierarchy7",
		  "TWRTriggerHierarchy8",
		  "TWRTriggerHierarchy9",
		  "TWRTriggerHierarchy10",
		  "TWRRawData0",
		  "TWRRawData1",
		  "TWRRawData2",
		  "TWRRawData3",
		  "TWRRawData4",
		  "TWRRawData5",
		  "TWRRawData6",
		  "TWRRawData7",
		  "TWRRawData8",
		  "TWRRawData9",
		  "TWRRawData10",
		  "TWRRawTimeCorrectionData0",
		  "TWRRawTimeCorrectionData1",
		  "TWRRawTimeCorrectionData2",
		  "TWRRawTimeCorrectionData3",
		  "TWRRawTimeCorrectionData4",
		  "TWRRawTimeCorrectionData5",
		  "TWRRawTimeCorrectionData6",
		  "TWRRawTimeCorrectionData7",
		  "TWRRawTimeCorrectionData8",
		  "TWRRawTimeCorrectionData9",
		  "TWRRawTimeCorrectionData10",
		  "TWRClockInfo0",
		  "TWRClockInfo1",
		  "TWRClockInfo2",
		  "TWRClockInfo3",
		  "TWRClockInfo4",
		  "TWRClockInfo5",
		  "TWRClockInfo6",
		  "TWRClockInfo7",
		  "TWRClockInfo8",
		  "TWRClockInfo9",
		  "TWRClockInfo10",
		  "TWRTWRLaunchSeriesMap0",
		  "TWRTWRLaunchSeriesMap1",
		  "TWRTWRLaunchSeriesMap2",
		  "TWRTWRLaunchSeriesMap3",
		  "TWRTWRLaunchSeriesMap4",
		  "TWRTWRLaunchSeriesMap5",
		  "TWRTWRLaunchSeriesMap6",
		  "TWRTWRLaunchSeriesMap7",
		  "TWRTWRLaunchSeriesMap8",
		  "TWRTWRLaunchSeriesMap9",
		  "TWRTWRLaunchSeriesMap10"]),
    )
#**************************************************************
# the BadDomList service
#**************************************************************
xmlf = workspace + "/BadDomList/resources/scripts/QueryConfiguration.xml"
tray.AddService("I3BadDomListFactory","bdlist")(
	("QFileName", xmlf ), 
	("BDListFile", outbdlfile), # optional output text file
	)

#**************************************************************
# Analysis-tree service
#**************************************************************
tray.AddService("I3RootTreeServiceFactory","service")(
	("TreeFileName", outrootfile)
	)

tray.AddModule("I3Muxer", "multiplex")

#**************************************************************
# EHE Filter
#**************************************************************
tray.AddModule("I3IcePickModule<FilterMaskFilter>", "skipEvents")(
    ("FilterNameList", ["EHEFilter_08"]),
    ("FilterResultName", "FilterMask"),
    ("DiscardEvents", True)
    )
tray.AddModule("Dump","dump");

#**************************************************************
# Get BadDomList
#**************************************************************
tray.AddModule("I3BadDomListModule","bdmodule")(
	("BadDomsListVector","BadDomsList"),
	) 
#***************************************************************
# Remove Bad DOMs
#***************************************************************
tray.AddModule("I3OMSelection<I3DOMLaunchSeries>","badoms_inice")(	
	("InputOMSelection","BadDomsList"),
	("OutputOMSelection","BadOMSelection_InIce"),
	("InputResponse","InIceRawData"),
	("OutputResponse","CleanInIceRawData")
	)
tray.AddModule("I3OMSelection<I3DOMLaunchSeries>","badoms_icetop")(	
	("InputOMSelection","BadDomsList"),
	("OutputOMSelection","BadOMSelection_IceTop"),
	("InputResponse","IceTopRawData"),
	("OutputResponse","CleanIceTopRawData")
	)
#***************************************************************
# Remove SLC-only readouts
#***************************************************************
tray.AddModule("I3LCCleaning","LCClean")(
	("InIceInput","CleanInIceRawData"),
	("InIceOutput","HLCInIceRawData")
	)
#***************************************************************
# Perform DOM calibration for InIce DOMs
#***************************************************************
tray.AddModule("I3DOMcalibrator","calibrate-inice")(
	("InputRawDataName","HLCInIceRawData"),
        ( "CalibrationMode",    1),
	)
#***************************************************************
# Portia
#***************************************************************
tray.AddModule("I3Portia","portiapulse")(
	("DataReadoutName",             "HLCInIceRawData"),
	("OutPortiaEventName",          "SummaryPulseInfo"),
	("MakeIceTopPulse",              False),
	("ATWDPulseSeriesName",         "ATWDPulseSeries"),
	("ATWDPortiaPulseName",         "ATWDPortiaPulse"),
	("ATWDWaveformName",            "CalibratedATWD"),
	("ATWDBaseLineOption",          "eheoptimized"),
	("FADCBaseLineOption",          "eheoptimized"),
	("ATWDThresholdCharge",         0.1*I3Units.pC),
	("ATWDLEThresholdAmplitude",    0.5*I3Units.mV),
	("UseFADC", True),
	("FADCPulseSeriesName",         "FADCPulseSeries"),
	("FADCPortiaPulseName",         "FADCPortiaPulse"),
	("FADCWaveformName",            "CalibratedFADC"),
	("FADCThresholdCharge",         0.1*I3Units.pC),
	("FADCLEThresholdAmplitude",    0.5*I3Units.mV),
	("MakeBestPulseSeries",         False),
	)
#***************************************************************
# Firstguess
#***************************************************************
tray.AddModule("I3EHEFirstGuess","reco")(
	("InputLaunchName",        "HLCInIceRawData"),
	("MinimumNumberPulseDom",  2),
	("OutputFirstguessName",   "OpheliaFirstGuess"),
	("InputPulseName1",        "ATWDPortiaPulse"),
	("InputPulseName2",        "FADCPortiaPulse"),
	("ChargeOption",           0),
	("LCOption",               True)
	)
#***************************************************************
# EHE Starting Event Identificator
#***************************************************************
tray.AddModule("I3OpheliaConvertFirstGuessTrack","fgconverter")(
	("InputOpheliaFGTrackName", "OpheliaFirstGuess"),
	("OutputParticleName", "OpheliaParticle"),
	)
tray.AddModule("I3EheStarLateralModule","starlat")(
	("useMCtruth",      False),
	("useRecoParticle", True),
	
	("inRecoParticleName", "OpheliaParticle"),
	("inAtwdPulseName", "ATWDPortiaPulse"),
	("inFadcPulseName", "FADCPortiaPulse"),
	("inBadDomListName", "BadDomsList"),
	
	("outEheStarLateralRecoName", "EheStarLateralFg"),
	("npe_zeroDom", 0.25),
	("npe_fitMin", 5),
	("npe_fillChMin", 1.0e4),
	)
tray.AddModule("I3EheStarFirstDomsModule","firstdoms")(
	("inPortiaPulseName", "ATWDPortiaPulse"),
	("outEheStarFirstDomsName", "EheStarFirstDoms"),
	)

#***************************************************************
# Analysis-tree
#***************************************************************
tray.AddModule("I3OpheliaConvertPortia","conv-portia")(
	("InputPortiaEventName", "SummaryPulseInfo"),
	("OutputPortiaEventMapName", "SummaryPulseInfoMap"),
	)
tray.AddModule("I3RootTreeModule","particle_tree1")(
	("ClassNames","I3Particle"),
	)
tray.AddModule("I3MapStringDoubleTreeModule","map_tree")(
	("KeyName","SummaryPulseInfoMap"),
	)
#***************************************************************
# Raw data eraser
#***************************************************************
tray.AddModule("I3OpheliaRawDataEraser","eraser")(
	("DataReadoutName", "HLCInIceRawData"),
	)
#***************************************************************
# Only keep selected frame objects
#***************************************************************
tray.AddModule( "Keep", "Cleanup" ) (
	("Keys", ["DrivingTime",
		  "I3EventHeader",
		  "I3TriggerHierarchy",
		  "JEBEventInfo",
		  "FilterMask",
		  "SyncPulseMap",
		  "IceTopVemPulseSeries",
		  "IcLinefit",
		  "IcLinefitParams",
		  "CascadeLinefit",
		  "CascadeLinefitParams",
		  "ToI",
		  "ToIParams",
		  "TrackLlhFit",
		  "TrackLlhFitFitParams",
		  "TrackParaboloidFit",
		  "TrackParaboloidFitFitParams",
		  "TrackBayesianFit",
		  "TrackBayesianFitFitParams",
		  "FirstSubtrackLlhFit",
		  "FirstSubtrackLlhFitFitParams",
		  "FirstSubtrackBayesianFit",
		  "FirstSubtrackBayesianFitFitParams",
		  "SecondSubtrackLlhFit",
		  "SecondSubtrackLlhFitFitParams",
		  "SecondSubtrackBayesianFit",
		  "SecondSubtrackBayesianFitFitParams",
		  "CascadeFirst",
		  "CascadeFirstParams",
		  "CascadeLlhVertexFit",
		  "CascadeLlhVertexFitParams",
		  "CascadeLlhEnergyFit",
		  "CascadeLlhEnergyFitParams",
		  "JamsFit",
		  "JamsFitParams",
		  "JamsQuality",
		  "JamsQualityParams",
		  "ShowerCOG",
		  "ShowerCOGParams",
		  "ShowerPlane",
		  "ShowerPlaneParams",
		  "ShowerEnergy",
		  "ShowerEnergyParams",
		  # EHE level3
		  "BadDomsList",
		  "HLCInIceRawData",
		  "CleanIceTopRawData",
		  "CleanInIceRawDataCleanedKeys",
		  "CleanIceTopRawDataCleanedKeys",
		  "CalibratedATWD",
		  "CalibratedFADC",
		  "ATWDPortiaPulse",
		  "FADCPortiaPulse",
		  "SummaryPulseInfo",
		  "OpheliaFirstGuess",
		  "EheStarLateralFg",
		  "EheStarFirstDoms"])
	)	
#***************************************************************
# Write physics frames
#***************************************************************
tray.AddModule("I3Writer","physwriter")(
	("Filename", outphysfile),
	("Streams",["Physics"]),
	)

tray.AddModule("TrashCan", "bye")

tray.Execute()
tray.Finish()
